########## R function: GVMP ##########

# For evaluating the function "G" that arises
# in variational message passing algorithms
# for the most fundamental semiparametric
# regression models. This function is defined
# in Section 2.7 of Wand (2017).

# Last changed: 18 MAY 2020

GVMP <- function(v,Q,r,s)
{
   # Set up is.wholenumber():
 
   is.wholenumber <- function(x,tol=sqrt(.Machine$double.eps))
      return(abs(x-round(x))<tol)
  
   # Check that v has valid length:

   dmn <- (sqrt(4*length(v) + 1) - 1)/2
   if(!is.wholenumber(dmn)) stop("Length of v not valid.")

   # Break up v into v1 and v2:

   v1 <- v[1:dmn]   ;   v2 <- v[-(1:dmn)]

   # Compute and return required quantity:

   vecinvv2 <- vecInverse(v2)
   tmpMat <- 0.25*solve(vecinvv2,(t(solve(vecinvv2,tcrossprod(v1)))-2*diag(dmn)))
   sumTerm <- sum(diag(Q%*%tmpMat)) + sum(r*solve(vecinvv2,v1)) + s
   
   return(-0.5*sumTerm)
}

############ End of GVMP ############
