########## R function: logmdigammaInv ##########

# For evaluation of the inverse of the
# "log minus digamma" function.

# Last changed: 18 MAY 2020

logmdigammaInv <- function(x)
{
   require(statmod)

   # Check for existence:

   if (x<=0)
      stop("x must be positive")
   
   if (x>0)
   {
      # Obtain function value via Newton-Raphson iteration:

      converged <- FALSE
      maxIter <- 100000
      toler <- 1e-10
      r <- 0.5/x
      rPrev <- r*(1 + 2*toler)
      itNum <- 0
      while(!converged)
      {
         itNum <- itNum + 1
         if (itNum>maxIter) 
         {  
            converged <- TRUE
            warning("Maximum number of iterations exceeded.")
         } 
         funVal <- logmdigamma(r) - x
         denom <- (1/r) - trigamma(r)
         r <- r - funVal/denom 
         relErr <- abs((r-rPrev)/rPrev)
         if (relErr<toler) 
            converged <- TRUE
       
         rPrev <- r
      }
   }

   return(r)
}  

############ End of logmdigammaInv ############
