########## R function: updateInvGWishPriorFrag ##########

# For the updating natural parameters of the factor to stochastic
# node message for the Inverse G-Wishart Prior fragment.
# This function corresponds to Algorithm 1 of 
# Maestrini & Wand (2021).

# Last changed: 08 OCT 2021

updateInvGWishPriorFrag <- function(GTheta,xiTheta,LambdaTheta)
{
   require("matrixcalc")

   # Obtain the dimension value:

   LambdaTheta <- as.matrix(LambdaTheta)
   dmn <- ncol(LambdaTheta)
   if (nrow(LambdaTheta)!=dmn)
      stop("LambdaTheta is not a square matrix.")

   # Obtain the required duplication matrix:

   if (dmn==1) Dd <- 1
   if (dmn>1) Dd <- duplication.matrix(dmn)
   
   # Set the factor to stochastic node message graph:

   GOUT <- GTheta

   # Set the factor to stochastic node message natural parameter:

   etaOUT <- -0.5*c((xiTheta+2),crossprod(Dd,vec(LambdaTheta)))

   # Return output list:

   return(list(G=GOUT,eta=etaOUT))
}

############ End of updateInvGWishPriorFrag ############
