########## R function: updatetLikFrag ##########

# For the updating natural parameters of factor to stochastic
# node messages for the t likelihood fragment.

# Last changed: 18 MAY 2020

updatetLikFrag <- function(y,A,etaIN)
{
  # The naming order for etaIN is:
  
  # 1. theta->p(y|theta,sigsq,a)
  # 2. p(y|theta,sigsq,a)->theta
  # 3. sigsq->p(y|theta,sigsq,a)
  # 4. p(y|theta,sigsq,a)->sigsq
  # 5. p(a|nu)->nu
  # 6. nu->p(a|nu)
  
  # The naming order for etaOUT is:
  
  # 1. "p(y|theta,sigsq,a)->theta",
  # 2. "p(y|theta,sigsq,a)->sigsq"
  # 3. "p(a|nu)->nu"
  
  # Obtain the sums of natural parameters along the
  # "theta", "sigsq" and "nu" edges:
  
  etaSUM1 <- etaIN[[1]] + etaIN[[2]]
  etaSUM2 <- etaIN[[3]] + etaIN[[4]]
  etaSUM3 <- etaIN[[5]] + etaIN[[6]]
  
  # Obtain the mu.q.sigsq update:
  
  mu.q.recip.sigsq <- (etaSUM2[1] + 1)/etaSUM2[2]
  
  # Obtain the mu.q.nu update:
  
  mu.q.nu <- 2*exp(logScrD(0,1,etaSUM3[1],etaSUM3[2])$modulus - logScrD(0,0,etaSUM3[1],etaSUM3[2])$modulus)

  # Obtain the auxiliary (`omega') updates:
  
  n <- length(y)
  
  onesVec <- rep(1,n)
  
  omega4 <- rep(NA,n)
  for (i in 1:n)
    omega4[i] <- GVMP(etaSUM1,tcrossprod(A[i,]),as.vector(A[i,])*y[i],(y[i]^2))
  
  omega5 <- ((mu.q.nu + 1)*onesVec)/(mu.q.nu*onesVec - 2*mu.q.recip.sigsq*omega4)
  
  # Update the natural parameters of the factor to stochastic node
  # messages:
  
  etaOUT[[1]] <- mu.q.recip.sigsq*c(crossprod(A,omega5*y),-0.5*vec(crossprod(A,omega5*A)))
  
  etaOUT[[2]] <- c((-0.5*n),GVMP(etaSUM1,crossprod(A,omega5*A),
                                 crossprod(A,omega5*y),sum((y^2)*omega5)))
  
  etaOUT[[3]] <- c(n,(n*digamma(0.5*(mu.q.nu+1)) - 
                        sum(log(0.5*mu.q.nu*onesVec - mu.q.recip.sigsq*omega4) + omega5)))
  
  return(etaOUT)
}

############ End of updatetLikFrag ############
